@@ -56,17 +56,18 @@
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkcombo.h>
-#include <gtk/gtkpixmap.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtktooltips.h>
+#include <gtk/gtkstock.h>
 
 #include <gphoto2/gphoto2-abilities-list.h>
 #include <gphoto2/gphoto2-setting.h>
 
-#include "support.h"
 #include "gtkam-error.h"
 #include "gtkam-close.h"
 #include "gtkam-port.h"
 #include "gtkam-status.h"
+#include "gtkam-util.h"
 
 struct _GtkamChooserPrivate
 {
@@ -79,11 +80,13 @@ struct _GtkamChooserPrivate
 
 	GtkWidget *ok;
 
+	GtkTooltips *tooltips;
+
 	gboolean needs_update;
 };
 
-#define PARENT_TYPE GTK_TYPE_DIALOG
-static GtkDialogClass *parent_class;
+#define PARENT_TYPE GTKAM_TYPE_DIALOG
+static GtkamDialogClass *parent_class;
 
 enum {
 	CAMERA_SELECTED,
@@ -92,8 +95,6 @@ enum {
 
 static guint signals[LAST_SIGNAL] = {0};
 
-#define gtk_marshal_NONE__POINTER_BOOL gtk_marshal_NONE__POINTER_INT
-
 static void
 gtkam_chooser_destroy (GtkObject *object)
 {
@@ -109,64 +110,124 @@ gtkam_chooser_destroy (GtkObject *object)
 		chooser->priv->il = NULL;
 	}
 
+	if (chooser->priv->tooltips) {
+		g_object_unref (G_OBJECT (chooser->priv->tooltips));
+		chooser->priv->tooltips = NULL;
+	}
+
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
 static void
-gtkam_chooser_finalize (GtkObject *object)
+gtkam_chooser_finalize (GObject *object)
 {
 	GtkamChooser *chooser = GTKAM_CHOOSER (object);
 
 	g_free (chooser->priv);
 
-	GTK_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
-gtkam_chooser_class_init (GtkamChooserClass *klass)
+gtkam_chooser_class_init (gpointer g_class, gpointer class_data)
 {
 	GtkObjectClass *object_class;
+	GObjectClass *gobject_class;
 
-	object_class = GTK_OBJECT_CLASS (klass);
+	object_class = GTK_OBJECT_CLASS (g_class);
 	object_class->destroy  = gtkam_chooser_destroy;
-	object_class->finalize = gtkam_chooser_finalize;
 
-	signals[CAMERA_SELECTED] = gtk_signal_new ("camera_selected",
-		GTK_RUN_LAST, object_class->type,
-		GTK_SIGNAL_OFFSET (GtkamChooserClass, camera_selected),
-		gtk_marshal_NONE__POINTER_BOOL, GTK_TYPE_NONE, 2,
-		GTK_TYPE_POINTER, GTK_TYPE_BOOL);
-	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
+	gobject_class = G_OBJECT_CLASS (g_class);
+	gobject_class->finalize = gtkam_chooser_finalize;
 
-	parent_class = gtk_type_class (PARENT_TYPE);
+	signals[CAMERA_SELECTED] = g_signal_new ("camera_selected",
+		G_TYPE_FROM_CLASS (g_class), G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GtkamChooserClass, camera_selected),
+		NULL, NULL, g_cclosure_marshal_VOID__POINTER, G_TYPE_NONE,
+		1, G_TYPE_POINTER);
+
+	parent_class = g_type_class_peek_parent (g_class);
 }
 
 static void
-gtkam_chooser_init (GtkamChooser *chooser)
+gtkam_chooser_init (GTypeInstance *instance, gpointer g_class)
 {
+	GtkamChooser *chooser = GTKAM_CHOOSER (instance);
+
 	chooser->priv = g_new0 (GtkamChooserPrivate, 1);
 }
 
-GtkType
+GType
 gtkam_chooser_get_type (void)
 {
-	static GtkType chooser_type = 0;
+	static GType type = 0;
 
-	if (!chooser_type) {
-		static const GtkTypeInfo chooser_info = {
-			"GtkamChooser",
-			sizeof (GtkamChooser),
-			sizeof (GtkamChooserClass),
-			(GtkClassInitFunc)  gtkam_chooser_class_init,
-			(GtkObjectInitFunc) gtkam_chooser_init,
-			NULL, NULL, NULL};
-		chooser_type = gtk_type_unique (PARENT_TYPE, &chooser_info);
+	if (!type) {
+		GTypeInfo ti;
+
+		memset (&ti, 0, sizeof (GTypeInfo));
+		ti.class_size     = sizeof (GtkamChooserClass);
+		ti.class_init     = gtkam_chooser_class_init;
+		ti.instance_size  = sizeof (GtkamChooser);
+		ti.instance_init  = gtkam_chooser_init;
+
+		type = g_type_register_static (PARENT_TYPE, "GtkamChooser",
+					       &ti, 0);
 	}
 
-	return (chooser_type);
+	return (type);
 }
 
-static Camera *
+#ifdef HAVE_GP_CAMERA_SET_TIMEOUT_FUNCS
+
+typedef struct _TimeoutData TimeoutData;
+struct _TimeoutData {
+        Camera *camera;
+        CameraTimeoutFunc func;
+};
+
+static gboolean
+timeout_func (gpointer data)
+{
+        TimeoutData *td = data;
+
+        td->func (td->camera, NULL);
+
+	/* Repeat forever */
+        return (TRUE);
+}
+
+static void
+timeout_destroy_notify (gpointer data)
+{
+        TimeoutData *td = data;
+
+        g_free (td);
+}
+
+static unsigned int
+start_timeout_func (Camera *camera, unsigned int timeout,
+                    CameraTimeoutFunc func, void *data)
+{
+        TimeoutData *td;
+
+        td = g_new0 (TimeoutData, 1);
+        td->camera = camera;
+        td->func = func;
+
+        return (gtk_timeout_add_full (timeout * 1000, timeout_func, NULL,
+                                      td, timeout_destroy_notify));
+}
+
+static void
+stop_timeout_func (Camera *camera, unsigned int id, void *data)
+{
+        gtk_timeout_remove (id);
+}
+
+#endif
+
+static GtkamCamera *
 gtkam_chooser_get_camera (GtkamChooser *chooser)
 {
 	GtkWidget *dialog, *status;
@@ -174,9 +235,10 @@ gtkam_chooser_get_camera (GtkamChooser *chooser)
 	Camera *camera;
 	CameraAbilities abilities;
 	const gchar *model, *port, *speed;
-	gchar *speed_number, *port_path, *right, *tmp;
+	gchar *port_path, *right, *tmp;
 	int m, p, r;
 	gboolean multi;
+	GtkamCamera *c;
 
 	model = gtk_entry_get_text (chooser->priv->entry_model);
 	port  = gtk_entry_get_text (chooser->priv->entry_port);
@@ -195,6 +257,10 @@ gtkam_chooser_get_camera (GtkamChooser *chooser)
 	}
 
 	gp_camera_new (&camera);
+#ifdef HAVE_GP_CAMERA_SET_TIMEOUT_FUNCS
+	gp_camera_set_timeout_funcs (camera, start_timeout_func,
+				     stop_timeout_func, NULL);
+#endif
 
 	/* Model? */
 	m = gp_abilities_list_lookup_model (chooser->priv->al, model);
@@ -218,9 +284,7 @@ gtkam_chooser_get_camera (GtkamChooser *chooser)
 	 * afterwards because other applications could need the camera, too.
 	 */
 	status = gtkam_status_new (_("Initializing camera..."));
-	gtk_widget_show (status);
-	gtk_box_pack_end (GTK_BOX (GTK_DIALOG (chooser)->vbox), status,
-			  FALSE, FALSE, 0);
+	gtkam_dialog_add_status (GTKAM_DIALOG (chooser), status);
 	r = gp_camera_init (camera, GTKAM_STATUS (status)->context->context);
 	if (multi)
 		gp_camera_exit (camera, NULL);
@@ -233,47 +297,34 @@ gtkam_chooser_get_camera (GtkamChooser *chooser)
 	default:
 		g_free (port_path);
 		dialog = gtkam_error_new (r, GTKAM_STATUS (status)->context,
-			GTK_WIDGET (chooser),
-			_("Could not initialize camera."));
+			NULL, _("Could not initialize camera."));
+		gtk_window_set_transient_for (GTK_WINDOW (dialog),
+					      GTK_WINDOW (chooser));
 		gtk_widget_show (dialog);
 		gp_camera_unref (camera);
-		gtk_object_destroy (GTK_OBJECT (status));
-		return (NULL);
+		camera = NULL;
 	}
 	gtk_object_destroy (GTK_OBJECT (status));
 
-	/* Remember the settings */
-	gp_setting_set ("gtkam", "model", (char*) model);
-	gp_setting_set ("gtkam", "path", port_path);
-	g_free (port_path);
-	if (!strcmp (speed, _("Best")))
-		speed_number = g_strdup ("0");
-	else
-		speed_number = g_strdup (speed);
-	gp_setting_set ("gtkam", "speed", (char*) speed_number);
-	g_free (speed_number);
-	if (multi)
-		gp_setting_set ("gtkam", "multi", "1");
-	else
-		gp_setting_set ("gtkam", "multi", "0");
+	c = gtkam_camera_new (camera, multi);
+	gp_camera_unref (camera);
 
-	return (camera);
+	return (c);
 }
 
 static void
 on_apply_clicked (GtkButton *button, GtkamChooser *chooser)
 {
-	Camera *camera;
+	GtkamCamera *camera;
 
 	if (!chooser->priv->needs_update)
 		return;
 
 	camera = gtkam_chooser_get_camera (chooser);
 	if (camera) {
-		gtk_signal_emit (GTK_OBJECT (chooser),
-			signals[CAMERA_SELECTED], camera,
-			GTK_TOGGLE_BUTTON (chooser->priv->check_multi)->active);
-		gp_camera_unref (camera);
+		g_signal_emit (G_OBJECT (chooser), signals[CAMERA_SELECTED],
+			       0, camera);
+		g_object_unref (G_OBJECT (camera));
 
 		chooser->priv->needs_update = FALSE;
 		gtk_widget_set_sensitive (chooser->apply_button, FALSE);
@@ -290,21 +341,16 @@ on_cancel_clicked (GtkButton *button, GtkamChooser *chooser)
 static void
 on_ok_clicked (GtkButton *button, GtkamChooser *chooser)
 {
-	Camera *camera;
+	GtkamCamera *camera;
 
 	if (chooser->priv->needs_update) {
-		gtk_widget_hide (GTK_WIDGET (chooser));
-		while (gtk_events_pending ())
-			gtk_main_iteration ();
 		camera = gtkam_chooser_get_camera (chooser);
 		if (camera) {
-			gtk_signal_emit (GTK_OBJECT (chooser),
-				signals[CAMERA_SELECTED], camera,
-				GTK_TOGGLE_BUTTON (chooser->priv->check_multi)->active);
-			gp_camera_unref (camera);
+			g_signal_emit (G_OBJECT (chooser),
+				       signals[CAMERA_SELECTED], 0, camera);
+			g_object_unref (G_OBJECT (camera));
 			gtk_object_destroy (GTK_OBJECT (chooser));
-		} else
-			gtk_widget_show (GTK_WIDGET (chooser));
+		}
 	} else
 		gtk_object_destroy (GTK_OBJECT (chooser));
 }
@@ -348,6 +394,8 @@ gtkam_chooser_update_for_model (GtkamChooser *chooser)
 
 	/* Get abilities of selected model */
 	model = gtk_entry_get_text (chooser->priv->entry_model);
+	if (!model || !*model)
+		return;
 	m = gp_abilities_list_lookup_model (chooser->priv->al, model);
 	result = gp_abilities_list_get_abilities (chooser->priv->al, m, &a);
 	if (result < 0) {
@@ -395,24 +443,23 @@ on_speed_changed (GtkEntry *entry, GtkamChooser *chooser)
 static void
 on_detect_clicked (GtkButton *button, GtkamChooser *chooser)
 {
-	GtkWidget *dialog, *status;
+	GtkWidget *d, *status;
 	CameraList list;
 	int result;
 	const char *name;
 
 	status = gtkam_status_new (_("Detecting cameras..."));
-	gtk_widget_show (status);
-	gtk_box_pack_end (GTK_BOX (GTK_DIALOG (chooser)->vbox), status,
-			  FALSE, FALSE, 0);
+	gtkam_dialog_add_status (GTKAM_DIALOG (chooser), status);
 	result = gp_abilities_list_detect (chooser->priv->al,
 		chooser->priv->il, &list,
 		GTKAM_STATUS (status)->context->context);
 	switch (result) {
 	case GP_OK:
 		if (!gp_list_count (&list)) {
-			dialog = gtkam_close_new (_("No cameras detected."),
-						  GTK_WIDGET (chooser));
-			gtk_widget_show (dialog);
+			d = gtkam_close_new (_("No cameras detected."));
+			gtk_window_set_transient_for (GTK_WINDOW (d),
+						      GTK_WINDOW (chooser));
+			gtk_widget_show (d);
 		} else {
 			//FIXME: Let user choose from the list
 			gp_list_get_name (&list, 0, &name);
@@ -424,10 +471,10 @@ on_detect_clicked (GtkButton *button, GtkamChooser *chooser)
 	case GP_ERROR_CANCEL:
 		break;
 	default:
-		dialog = gtkam_error_new (result,
+		d = gtkam_error_new (result,
 			GTKAM_STATUS (status)->context, GTK_WIDGET (chooser),
 			_("Could not detect any cameras."));
-		gtk_widget_show (dialog);
+		gtk_widget_show (d);
 		break;
 	}
 	gtk_object_destroy (GTK_OBJECT (status));
@@ -529,7 +576,7 @@ on_add_clicked (GtkButton *button, GtkamChooser *chooser)
 
 	port = gtkam_port_new (GTK_WIDGET (chooser));
 	gtk_widget_show (port);
-	gtk_signal_connect (GTK_OBJECT (port), "port_added",
+	g_signal_connect (GTK_OBJECT (port), "port_added",
 			    GTK_SIGNAL_FUNC (on_port_added), chooser);
 }
 
@@ -537,15 +584,14 @@ GtkWidget *
 gtkam_chooser_new (void)
 {
 	GtkamChooser *chooser;
-	GtkWidget *table, *label, *button, *combo, *hbox, *vbox, *check;
-	GtkWidget *image;
-	GdkPixmap *pixmap;
-	GdkBitmap *bitmap;
-	GdkPixbuf *pixbuf;
-	char port[1024], speed[1024], model[1024], multi[1024];
-	gboolean p, s, m, mult;
+	GtkWidget *table, *label, *button, *combo, *vbox, *check;
+	gchar *t;
 
-	chooser = gtk_type_new (GTKAM_TYPE_CHOOSER);
+	chooser = g_object_new (GTKAM_TYPE_CHOOSER, NULL);
+
+	chooser->priv->tooltips = gtk_tooltips_new ();
+	g_object_ref (G_OBJECT (chooser->priv->tooltips));
+	gtk_object_sink (GTK_OBJECT (chooser->priv->tooltips));
 
 	gp_abilities_list_new (&(chooser->priv->al));
 	gp_abilities_list_load (chooser->priv->al, NULL);
@@ -553,36 +599,15 @@ gtkam_chooser_new (void)
 	gp_port_info_list_new (&(chooser->priv->il));
 	gp_port_info_list_load (chooser->priv->il);
 
-	gtk_window_set_title (GTK_WINDOW (chooser), _("Select Camera"));
+	t = g_locale_to_utf8 (_("Select Camera"), -1, NULL, NULL, NULL);
+	gtk_window_set_title (GTK_WINDOW (chooser), t);
+	g_free (t);
 	gtk_container_set_border_width (GTK_CONTAINER (chooser), 5);
-	gtk_window_set_policy (GTK_WINDOW (chooser), TRUE, TRUE, TRUE);
-	gtk_signal_connect (GTK_OBJECT (chooser), "delete_event",
-			    GTK_SIGNAL_FUNC (gtk_object_destroy), NULL);
-
-	hbox = gtk_hbox_new (FALSE, 10);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (chooser)->vbox), hbox,
-			    TRUE, TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);
-
-	pixbuf = gdk_pixbuf_new_from_file (IMAGE_DIR "/gtkam-camera.png");
-	if (!pixbuf)
-		g_warning ("Could not load " IMAGE_DIR "/gtkam-camera.png");
-	else {
-		gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, &bitmap, 127);
-		gdk_pixbuf_unref (pixbuf);
-		image = gtk_pixmap_new (pixmap, bitmap);
-		if (pixmap)
-			gdk_pixmap_unref (pixmap);
-		if (bitmap)
-			gdk_bitmap_unref (bitmap);
-		gtk_widget_show (image);
-		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-	}
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox);
-	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (GTKAM_DIALOG (chooser)->vbox),
+			    vbox, TRUE, TRUE, 0);
 
 	table = gtk_table_new (3, 3, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 5);
@@ -600,13 +625,19 @@ gtkam_chooser_new (void)
 	gtk_table_attach_defaults (GTK_TABLE (table), combo, 1, 2, 0, 1);
 	chooser->priv->combo_model = GTK_COMBO (combo);
 	chooser->priv->entry_model = GTK_ENTRY (GTK_COMBO (combo)->entry);
+#if 0
 	gtk_entry_set_editable (chooser->priv->entry_model, FALSE);
+#endif
 
-	button = gtk_button_new_with_label (_("Detect"));
+	t = g_locale_to_utf8 (_("Detect"), -1, NULL, NULL, NULL);
+	button = gtk_button_new_with_label (t);
+	g_free (t);
 	gtk_widget_show (button);
 	gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 0, 1);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+	g_signal_connect (GTK_OBJECT (button), "clicked",
 			    GTK_SIGNAL_FUNC (on_detect_clicked), chooser);
+	gtk_tooltips_set_tip (chooser->priv->tooltips, button,
+			      _("Detect USB camera"), NULL);
 
 	label = gtk_label_new (_("Port:"));
 	gtk_widget_show (label);
@@ -618,12 +649,14 @@ gtkam_chooser_new (void)
 	gtk_widget_set_sensitive (combo, FALSE);
 	chooser->priv->entry_port = GTK_ENTRY (GTK_COMBO (combo)->entry);
 	chooser->priv->combo_port = GTK_COMBO (combo);
+#if 0
 	gtk_entry_set_editable (chooser->priv->entry_port, FALSE);
+#endif
 
-	button = gtk_button_new_with_label (_("Add"));
+	button = gtk_button_new_from_stock (GTK_STOCK_ADD);
 	gtk_widget_set_sensitive (button, FALSE);
 	gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 1, 2);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+	g_signal_connect (GTK_OBJECT (button), "clicked",
 			    GTK_SIGNAL_FUNC (on_add_clicked), chooser);
 	chooser->priv->button_add = button;
 
@@ -637,133 +670,67 @@ gtkam_chooser_new (void)
 	chooser->priv->entry_speed = GTK_ENTRY (GTK_COMBO (combo)->entry);
 	gtk_entry_set_text (chooser->priv->entry_speed, _("Best"));
 	chooser->priv->combo_speed = GTK_COMBO (combo);
+#if 0
 	gtk_entry_set_editable (chooser->priv->entry_speed, FALSE);
+#endif
 
-	check = gtk_check_button_new_with_label (_("Allow multiple frontends"));
+	t = g_locale_to_utf8 (_("Allow multiple frontends"), -1,
+			      NULL, NULL, NULL);
+	check = gtk_check_button_new_with_label (t);
+	g_free (t);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), FALSE);
 	gtk_table_attach_defaults (GTK_TABLE (table), check, 0, 2, 3, 4);
-	gtk_signal_connect (GTK_OBJECT (check), "toggled",
+	g_signal_connect (GTK_OBJECT (check), "toggled",
 			    GTK_SIGNAL_FUNC (on_multi_toggled), chooser);
 	chooser->priv->check_multi = check;
 
-	button = gtk_toggle_button_new_with_label (_("Enhanced"));
+	t = g_locale_to_utf8 (_("Enhanced"), -1, NULL, NULL, NULL);
+	button = gtk_toggle_button_new_with_label (t);
+	g_free (t);
 	gtk_widget_show (button);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (chooser)->action_area),
 			   button);
-	gtk_signal_connect (GTK_OBJECT (button), "toggled",
+	g_signal_connect (GTK_OBJECT (button), "toggled",
 			    GTK_SIGNAL_FUNC (on_more_options_toggled), chooser);
 
-	button = gtk_button_new_with_label (_("Ok"));
+	button = gtk_button_new_from_stock (GTK_STOCK_OK);
 	gtk_widget_show (button);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (chooser)->action_area),
 			   button);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+	g_signal_connect (GTK_OBJECT (button), "clicked",
 			    GTK_SIGNAL_FUNC (on_ok_clicked), chooser);
 	gtk_widget_grab_focus (button);
 	chooser->priv->ok = button;
 
-	chooser->apply_button = gtk_button_new_with_label (_("Apply"));
+	chooser->apply_button = gtk_button_new_from_stock (GTK_STOCK_APPLY);
 	gtk_widget_show (chooser->apply_button);
 	gtk_widget_set_sensitive (chooser->apply_button, FALSE);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (chooser)->action_area),
 			   chooser->apply_button);
-	gtk_signal_connect (GTK_OBJECT (chooser->apply_button), "clicked",
+	g_signal_connect (GTK_OBJECT (chooser->apply_button), "clicked",
 			    GTK_SIGNAL_FUNC (on_apply_clicked), chooser);
 
-	button = gtk_button_new_with_label (_("Cancel"));
+	button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
 	gtk_widget_show (button);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (chooser)->action_area),
 			   button);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+	g_signal_connect (GTK_OBJECT (button), "clicked",
 			    GTK_SIGNAL_FUNC (on_cancel_clicked), chooser);
 
 	/* Fill the model combo with all models */
 	gtkam_chooser_set_camera_mask (chooser, GP_OPERATION_NONE);
 
-	gtk_signal_connect (GTK_OBJECT (chooser->priv->entry_model), "changed",
-			    GTK_SIGNAL_FUNC (on_model_changed), chooser);
-	gtk_signal_connect (GTK_OBJECT (chooser->priv->entry_port), "changed",
-			    GTK_SIGNAL_FUNC (on_port_changed), chooser);
-	gtk_signal_connect (GTK_OBJECT (chooser->priv->entry_speed), "changed",
-			    GTK_SIGNAL_FUNC (on_speed_changed), chooser);
-
-	/* Remember what we update */
-	mult = p = m = s = FALSE;
-
-	/* Do we have a model? */
-	if ((gp_setting_get ("gtkam", "model", model) == GP_OK) ||
-	    (gp_setting_get ("gphoto2", "model", model) == GP_OK)) {
-		m = TRUE;
-		gtk_entry_set_text (chooser->priv->entry_model, model);
-	}
-
-	/* Do we have a port? */
-	if ((gp_setting_get ("gtkam", "path", port) == GP_OK) ||
-	    (gp_setting_get ("gphoto2", "port", port) == GP_OK)) {
-		int n, r;
-		GPPortInfo info;
-		guint i;
-		gchar *name;
-
-		n = gp_port_info_list_count (chooser->priv->il);
-		for (i = 0; i < n; i++) {
-			gp_port_info_list_get_info (chooser->priv->il, i,
-						    &info);
-			if (!strcmp (info.name, port) ||
-			    !strcmp (info.path, port)) {
-				name = g_strdup_printf ("%s (%s)",
-							info.name, info.path);
-				gtk_entry_set_text (chooser->priv->entry_port,
-						    name);
-				p = TRUE;
-				g_free (name);
-				break;
-			}
-		}
-		if (i == n) {
-			r = gp_port_info_list_lookup_path (chooser->priv->il,
-							   port);
-			if (r >= 0) {
-				gp_port_info_list_get_info (chooser->priv->il,
-							    i, &info);
-				name = g_strdup_printf ("%s (%s)",
-							info.name, info.path);
-				gtk_entry_set_text (chooser->priv->entry_port,
-						    name);
-				p = TRUE;
-				g_free (name);
-			}
-		}
-	}
-
-	/* Do we have a speed? */
-	if (gp_setting_get ("gtkam", "speed", speed) == GP_OK) {
-		if (atoi (speed))
-			gtk_entry_set_text (chooser->priv->entry_speed, speed);
-		else
-			gtk_entry_set_text (chooser->priv->entry_speed,
-					    _("Best"));
-		s = TRUE;
-	}
-
-	/* Do we have the multi option? */
-	if (gp_setting_get ("gtkam", "multi", multi) == GP_OK) {
-		if (atoi (multi))
-			gtk_toggle_button_set_active (
-				GTK_TOGGLE_BUTTON (chooser->priv->check_multi),
-				TRUE);
-		else
-			gtk_toggle_button_set_active (
-				GTK_TOGGLE_BUTTON (chooser->priv->check_multi),
-				FALSE);
-		mult = TRUE;
-	}
+	g_signal_connect (G_OBJECT (chooser->priv->entry_model), "changed",
+			  G_CALLBACK (on_model_changed), chooser);
+	g_signal_connect (G_OBJECT (chooser->priv->entry_port), "changed",
+			  G_CALLBACK (on_port_changed), chooser);
+	g_signal_connect (G_OBJECT (chooser->priv->entry_speed), "changed",
+			  G_CALLBACK (on_speed_changed), chooser);
 
 	gtkam_chooser_update_for_model (chooser);
 
-	chooser->priv->needs_update = !m || !p || !s || !mult;
-	gtk_widget_set_sensitive (chooser->apply_button,
-				  chooser->priv->needs_update);
+	chooser->priv->needs_update = TRUE;
+	gtk_widget_set_sensitive (chooser->apply_button, TRUE);
 
 	return (GTK_WIDGET (chooser));
 }
@@ -828,3 +795,28 @@ gtkam_chooser_set_port_mask (GtkamChooser *chooser, GPPortType types)
 
 	gtkam_chooser_set_port_list (chooser, list);
 }
+
+void
+gtkam_chooser_set_camera (GtkamChooser *chooser, GtkamCamera *camera)
+{
+	CameraAbilities a;
+	GPPortInfo info;
+	gchar *full_info;
+
+	g_return_if_fail (GTKAM_IS_CHOOSER (chooser));
+	g_return_if_fail (GTKAM_IS_CAMERA (camera));
+
+	gp_camera_get_abilities (camera->camera, &a);
+	gp_camera_get_port_info (camera->camera, &info);
+
+	full_info = g_strdup_printf ("%s (%s)", info.name, info.path);
+	gtk_entry_set_text (chooser->priv->entry_port, full_info);
+	g_free (full_info);
+
+	gtk_entry_set_text (chooser->priv->entry_model, a.model);
+	gtk_toggle_button_set_active (
+		GTK_TOGGLE_BUTTON (chooser->priv->check_multi), camera->multi);
+
+	chooser->priv->needs_update = FALSE;
+	gtk_widget_set_sensitive (chooser->apply_button, FALSE);
+}
