@@ -19,6 +19,7 @@
  */
 
 #include <config.h>
+#include <math.h>
 #include "gtkam-config.h"
 
 #ifdef ENABLE_NLS
@@ -44,7 +45,9 @@
 #include <string.h>
 #include <time.h>
 
+#include <gtk/gtkstock.h>
 #include <gtk/gtktooltips.h>
+#include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtknotebook.h>
 #include <gtk/gtkhscale.h>
@@ -74,14 +77,12 @@ struct _GtkamConfigPrivate
 
 	GHashTable *hash;
 
-	Camera       *camera;
+	GtkamCamera  *camera;
 	CameraWidget *config;
-
-	gboolean multi;
 };
 
-#define PARENT_TYPE GTK_TYPE_DIALOG
-static GtkDialogClass *parent_class;
+#define PARENT_TYPE GTKAM_TYPE_DIALOG
+static GtkamDialogClass *parent_class;
 
 static void
 gtkam_config_destroy (GtkObject *object)
@@ -89,7 +90,7 @@ gtkam_config_destroy (GtkObject *object)
 	GtkamConfig *config = GTKAM_CONFIG (object);
 
 	if (config->priv->camera) {
-		gp_camera_unref (config->priv->camera);
+		g_object_unref (config->priv->camera);
 		config->priv->camera = NULL;
 	}
 
@@ -104,7 +105,7 @@ gtkam_config_destroy (GtkObject *object)
 	}
 
 	if (config->priv->tooltips) {
-		gtk_object_unref (GTK_OBJECT (config->priv->tooltips));
+		g_object_unref (G_OBJECT (config->priv->tooltips));
 		config->priv->tooltips = NULL;
 	}
 
@@ -112,52 +113,61 @@ gtkam_config_destroy (GtkObject *object)
 }
 
 static void
-gtkam_config_finalize (GtkObject *object)
+gtkam_config_finalize (GObject *object)
 {
 	GtkamConfig *config = GTKAM_CONFIG (object);
 
 	g_free (config->priv);
 
-	GTK_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
-gtkam_config_class_init (GtkamConfigClass *klass)
+gtkam_config_class_init (gpointer g_class, gpointer class_data)
 {
 	GtkObjectClass *object_class;
+	GObjectClass *gobject_class;
 
-	object_class = GTK_OBJECT_CLASS (klass);
+	object_class = GTK_OBJECT_CLASS (g_class);
 	object_class->destroy  = gtkam_config_destroy;
-	object_class->finalize = gtkam_config_finalize;
+	
+	gobject_class = G_OBJECT_CLASS (g_class);
+	gobject_class->finalize = gtkam_config_finalize;
 
-	parent_class = gtk_type_class (PARENT_TYPE);
+	parent_class = g_type_class_peek_parent (g_class);
 }
 
 static void
-gtkam_config_init (GtkamConfig *config)
+gtkam_config_init (GTypeInstance *instance, gpointer g_class)
 {
+	GtkamConfig *config = GTKAM_CONFIG (instance);
+
 	config->priv = g_new0 (GtkamConfigPrivate, 1);
 	config->priv->hash = g_hash_table_new (g_direct_hash, g_direct_equal);
 	config->priv->tooltips = gtk_tooltips_new ();
+	g_object_ref (G_OBJECT (config->priv->tooltips));
+	gtk_object_sink (GTK_OBJECT (config->priv->tooltips));
 }
 
-GtkType
+GType
 gtkam_config_get_type (void)
 {
-	static GtkType config_type = 0;
+	static GType type = 0;
 
-	if (!config_type) {
-		static const GtkTypeInfo config_info = {
-			"GtkamConfig",
-			sizeof (GtkamConfig),
-			sizeof (GtkamConfigClass),
-			(GtkClassInitFunc)  gtkam_config_class_init,
-			(GtkObjectInitFunc) gtkam_config_init,
-			NULL, NULL, NULL};
-		config_type = gtk_type_unique (PARENT_TYPE, &config_info);
+	if (!type) {
+		GTypeInfo ti;
+
+		memset (&ti, 0, sizeof (GTypeInfo)); 
+		ti.class_size     = sizeof (GtkamConfigClass);
+		ti.class_init     = gtkam_config_class_init;
+		ti.instance_size  = sizeof (GtkamConfig);
+		ti.instance_init  = gtkam_config_init;
+
+		type = g_type_register_static (PARENT_TYPE, "GtkamConfig",
+					       &ti, 0);
 	}
 
-	return (config_type);
+	return (type);
 }
 
 static void
@@ -170,10 +180,10 @@ gtkam_config_apply (GtkamConfig *config)
 	gtk_widget_show (status);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (config)->vbox), status,
 			    FALSE, FALSE, 0);
-	result = gp_camera_set_config (config->priv->camera,
+	result = gp_camera_set_config (config->priv->camera->camera,
 		config->priv->config, GTKAM_STATUS (status)->context->context);
-	if (config->priv->multi)
-		gp_camera_exit (config->priv->camera, NULL);
+	if (config->priv->camera->multi)
+		gp_camera_exit (config->priv->camera->camera, NULL);
 	switch (result) {
 	case GP_OK:
 		break;
@@ -193,6 +203,7 @@ gtkam_config_close (GtkamConfig *config)
 {
 	while (gtk_events_pending ())
 		gtk_main_iteration ();
+
 	gtk_object_destroy (GTK_OBJECT (config));
 }
 
@@ -218,7 +229,7 @@ on_config_close_clicked (GtkButton *button, GtkamConfig *config)
 static GtkWidget *
 create_page (GtkamConfig *config, CameraWidget *widget)
 {
-	GtkWidget *label, *vbox;
+	GtkWidget *label, *vbox, *scrolled;
 	int id;
 	const char *l = NULL, *info = NULL;
 
@@ -237,12 +248,28 @@ create_page (GtkamConfig *config, CameraWidget *widget)
 		label = gtk_label_new (_("General Settings"));
 	gtk_widget_show (label);
 
-	/* VBox */
-	vbox = gtk_vbox_new (FALSE, 10);
-	gtk_widget_show (vbox);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
-	gtk_notebook_append_page (GTK_NOTEBOOK (config->priv->notebook), vbox,
-				  label);
+	if (gp_widget_count_children (widget) > 4) {
+		scrolled = gtk_scrolled_window_new (NULL, NULL);
+		gtk_widget_show (scrolled);
+		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
+				GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+		gtk_container_set_border_width (GTK_CONTAINER (scrolled), 5);
+		gtk_notebook_append_page (
+			GTK_NOTEBOOK (config->priv->notebook),
+			scrolled, label);
+		vbox = gtk_vbox_new (FALSE, 10);
+		gtk_widget_show (vbox);
+		gtk_scrolled_window_add_with_viewport (
+				GTK_SCROLLED_WINDOW (scrolled), vbox);
+		gtk_container_set_border_width (GTK_CONTAINER (vbox), 2);
+	} else {
+		vbox = gtk_vbox_new (FALSE, 10);
+		gtk_widget_show (vbox);
+		gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+		gtk_notebook_append_page (
+			GTK_NOTEBOOK (config->priv->notebook), 
+			vbox, label);
+	}
 
 	if (widget) {
 		gp_widget_get_id (widget, &id);
@@ -274,7 +301,7 @@ on_radio_button_toggled (GtkToggleButton *toggle, CameraWidget *widget)
 		return;
 
 	gp_widget_get_value (widget, &value);
-	value_new = gtk_object_get_data (GTK_OBJECT (toggle), "value");
+	value_new = g_object_get_data (G_OBJECT (toggle), "value");
 	if (!value || strcmp (value, value_new))
 		gp_widget_set_value (widget, value_new);
 }
@@ -290,7 +317,7 @@ on_button_clicked (GtkButton *button, CameraWidget *widget)
 	config = GTKAM_CONFIG (gtk_widget_get_ancestor (GTK_WIDGET (button),
 							GTKAM_TYPE_CONFIG));
 	gp_widget_get_value (widget, &callback);
-	result = callback (config->priv->camera, widget, NULL);
+	result = callback (config->priv->camera->camera, widget, NULL);
 	if (result != GP_OK) {
 		dialog = gtkam_error_new (result, NULL, GTK_WIDGET (config),
 			_("Could not execute command."));
@@ -301,12 +328,13 @@ on_button_clicked (GtkButton *button, CameraWidget *widget)
 static void
 on_entry_changed (GtkEntry *entry, CameraWidget *widget)
 {
-	char *value = NULL, *value_new;
+	char *value = NULL;
+	const char *value_new;
 
 	gp_widget_get_value (widget, &value);
 	value_new = gtk_entry_get_text (entry);
 	if (!value || strcmp (value, value_new))
-		gp_widget_set_value (widget, value_new);
+		gp_widget_set_value (widget, (void *) value_new);
 }
 
 static void
@@ -326,9 +354,17 @@ adjust_time (GtkCalendar *calendar, GtkamClock *clock, CameraWidget *widget)
 	struct tm tm;
 	time_t time;
 
+	memset (&tm, 0, sizeof (struct tm));
 	gtkam_clock_get (clock, (guchar*) &tm.tm_hour,
 			 (guchar*) &tm.tm_min, (guchar*) &tm.tm_sec);
 	gtk_calendar_get_date (calendar, &tm.tm_year, &tm.tm_mon, &tm.tm_mday);
+
+	/*
+	 * According to Patrick Mansfield <patman@aracnet.com>, tm_year
+	 * starts counting at 1900, whereas gtk_calendar starts counting at 0.
+	 */
+	tm.tm_year = tm.tm_year - 1900;
+
 	time = mktime (&tm);
 	gp_widget_set_value (widget, (int*) &time);
 }
@@ -338,7 +374,7 @@ on_day_selected (GtkCalendar *calendar, CameraWidget *widget)
 {
 	GtkamClock *clock;
 
-	clock = GTKAM_CLOCK (gtk_object_get_data (GTK_OBJECT (calendar),
+	clock = GTKAM_CLOCK (g_object_get_data (G_OBJECT (calendar),
 						  "clock"));
 	adjust_time (calendar, clock, widget);
 }
@@ -366,7 +402,7 @@ on_clock_next_day (GtkamClock *clock)
 {
 	GtkCalendar *calendar;
 
-	calendar = GTK_CALENDAR (gtk_object_get_data (GTK_OBJECT (clock),
+	calendar = GTK_CALENDAR (g_object_get_data (G_OBJECT (clock),
 						      "calendar"));
 	adjust_day (calendar, 1);
 }
@@ -376,7 +412,7 @@ on_clock_previous_day (GtkamClock *clock)
 {
 	GtkCalendar *calendar;
 
-	calendar = GTK_CALENDAR (gtk_object_get_data (GTK_OBJECT (clock),
+	calendar = GTK_CALENDAR (g_object_get_data (G_OBJECT (clock),
 						      "calendar"));
 	adjust_day (calendar, -1);
 }
@@ -386,11 +422,51 @@ on_clock_set (GtkamClock *clock, CameraWidget *widget)
 {
 	GtkCalendar *calendar;
 
-	calendar = GTK_CALENDAR (gtk_object_get_data (GTK_OBJECT (clock),
+	calendar = GTK_CALENDAR (g_object_get_data (G_OBJECT (clock),
 						      "calendar"));
 	adjust_time (calendar, clock, widget);
 }
 
+static int 
+get_digits (double d)
+{
+	/*
+	 * Returns the number of non-zero digits to the right of the decimal
+	 * point, up to a max of 4.
+	 */
+	double x;
+	int i;
+	for (i = 0, x = d * 1.0; i < 4;  i++) {
+
+		/*
+		 * The small number really depends on how many digits (4)
+		 * we are checking for, but as long as it's small enough
+		 * this works fine. We need the "<" as the floating point
+		 * arithemtic does not always give an exact 0.0 (and can
+		 * even give a -0.0).
+		 */
+		if (fabs (x - floor (x)) < 0.000001) 
+			return(i);
+		x = x * 10.0;
+	}
+	return(i);
+}
+
+static void
+on_set_system_time_clicked (GtkButton *button, GtkamClock *clock)
+{
+	GtkCalendar *c;
+	time_t t;
+	struct tm *tm;
+
+	c = GTK_CALENDAR (g_object_get_data (G_OBJECT (clock), "calendar"));
+	t = time (NULL);
+	tm = localtime (&t);
+	gtk_calendar_select_month (c, tm->tm_mon, tm->tm_year + 1900);
+	gtk_calendar_select_day (c, tm->tm_mday);
+	gtkam_clock_set (clock, tm->tm_hour, tm->tm_min, tm->tm_sec);
+}
+
 static void
 create_widgets (GtkamConfig *config, CameraWidget *widget)
 {
@@ -401,8 +477,8 @@ create_widgets (GtkamConfig *config, CameraWidget *widget)
 	int value_int;
 	float value_float, min, max, increment;
 	const char *choice;
-	GtkWidget *vbox, *button, *gtk_widget = NULL, *frame, *calendar;
-	GtkWidget *clock;
+	GtkWidget *vbox, *button, *gw = NULL, *frame, *calendar;
+	GtkWidget *clock, *hbox;
 	GtkObject *adjustment;
 	GSList *group = NULL;
 	GList *options = NULL;
@@ -436,39 +512,40 @@ create_widgets (GtkamConfig *config, CameraWidget *widget)
 	
 	case GP_WIDGET_BUTTON:
 
-		gtk_widget = gtk_vbox_new (FALSE, 10);
-		gtk_container_set_border_width (GTK_CONTAINER (gtk_widget), 10);
+		gw = gtk_vbox_new (FALSE, 10);
+		gtk_container_set_border_width (GTK_CONTAINER (gw), 10);
 		button = gtk_button_new_with_label (_(label));
 		gtk_widget_show (button);
-		gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		g_signal_connect (GTK_OBJECT (button), "clicked",
 				GTK_SIGNAL_FUNC (on_button_clicked), widget);
-		gtk_container_add (GTK_CONTAINER (gtk_widget), button);
+		gtk_container_add (GTK_CONTAINER (gw), button);
 		if (strlen (info))
 			gtk_tooltips_set_tip (config->priv->tooltips,
 				button, info, NULL);
 		else
-			gtk_tooltips_set_tip (config->priv->tooltips,
-				button,
-				_("No additional information available. (2)"), NULL);
+			gtk_tooltips_set_tip (config->priv->tooltips, button,
+				_("No additional information available. (2)"),
+				NULL);
 		break;
 
 	case GP_WIDGET_DATE:
 
 		gp_widget_get_value (widget, &value_int);
-		gtk_widget = gtk_vbox_new (FALSE, 5);
-		gtk_container_set_border_width (GTK_CONTAINER (gtk_widget), 5);
+		gw = gtk_vbox_new (FALSE, 5);
+		gtk_container_set_border_width (GTK_CONTAINER (gw), 5);
 
 		/* Create the calendar */
 		calendar = gtk_calendar_new ();
 		gtk_widget_show (calendar);
-		gtk_box_pack_start (GTK_BOX (gtk_widget), calendar, FALSE,
-				    FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (gw), calendar, FALSE, FALSE, 0);
 
 		/* Create the clock */
 		clock = gtkam_clock_new ();
 		gtk_widget_show (clock);
-		gtk_box_pack_start (GTK_BOX (gtk_widget), clock,
-				    FALSE, FALSE, 0);
+		hbox = gtk_hbox_new (5, FALSE);
+		gtk_widget_show (hbox);
+		gtk_box_pack_start (GTK_BOX (gw), hbox, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (hbox), clock, FALSE, FALSE, 0);
 
 		/* Set date & time */
 		tm = localtime ((time_t*) &value_int);
@@ -479,17 +556,25 @@ create_widgets (GtkamConfig *config, CameraWidget *widget)
 				 tm->tm_sec);
 
 		/* We need clock and calendar together */
-		gtk_object_set_data (GTK_OBJECT (clock), "calendar", calendar);
-		gtk_object_set_data (GTK_OBJECT (calendar), "clock", clock);
+		g_object_set_data (G_OBJECT (clock), "calendar", calendar);
+		g_object_set_data (G_OBJECT (calendar), "clock", clock);
+
+		/* Make it easy for users to select the system's time */
+		button = gtk_button_new_with_label (_("Set to system's time"));
+		gtk_widget_show (button);
+		gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+		g_signal_connect (G_OBJECT (button), "clicked",
+				  G_CALLBACK (on_set_system_time_clicked),
+				  clock);
 
 		/* Connect the signals */
-		gtk_signal_connect (GTK_OBJECT (clock), "next_day",
+		g_signal_connect (GTK_OBJECT (clock), "next_day",
 			GTK_SIGNAL_FUNC (on_clock_next_day), NULL);
-		gtk_signal_connect (GTK_OBJECT (clock), "previous_day",
+		g_signal_connect (GTK_OBJECT (clock), "previous_day",
 			GTK_SIGNAL_FUNC (on_clock_previous_day), NULL);
-		gtk_signal_connect (GTK_OBJECT (clock), "set",
+		g_signal_connect (GTK_OBJECT (clock), "set",
 			GTK_SIGNAL_FUNC (on_clock_set), widget);
-		gtk_signal_connect (GTK_OBJECT (calendar), "day_selected",
+		g_signal_connect (GTK_OBJECT (calendar), "day_selected",
 			GTK_SIGNAL_FUNC (on_day_selected), widget);
 
 		break;
@@ -497,18 +582,18 @@ create_widgets (GtkamConfig *config, CameraWidget *widget)
 	case GP_WIDGET_TEXT:
 
 		gp_widget_get_value (widget, &value_char);
-		gtk_widget = gtk_entry_new ();
+		gw = gtk_entry_new ();
 		if (value_char)
-			gtk_entry_set_text (GTK_ENTRY (gtk_widget),
+			gtk_entry_set_text (GTK_ENTRY (gw),
 					    _(value_char));
-		gtk_signal_connect (GTK_OBJECT (gtk_widget), "changed",
+		g_signal_connect (GTK_OBJECT (gw), "changed",
 				GTK_SIGNAL_FUNC (on_entry_changed), widget);
 		if (strlen (info))
 			gtk_tooltips_set_tip (config->priv->tooltips,
-				gtk_widget, info, NULL);
+				gw, info, NULL);
 		else
 			gtk_tooltips_set_tip (config->priv->tooltips,
-				gtk_widget,
+				gw,
 				_("No additional information available. (3)"), NULL);
 		break;
 	
@@ -517,59 +602,63 @@ create_widgets (GtkamConfig *config, CameraWidget *widget)
 		gp_widget_get_value (widget, &value_float);
 		gp_widget_get_range (widget, &min, &max, &increment);
 		adjustment = gtk_adjustment_new (value_float, min, max,
-						 increment, 0, 0);
-		gtk_signal_connect (adjustment, "value_changed",
+						 increment, 
+						 MAX((max - min) / 20.0, 
+						     increment),
+						 0);
+		g_signal_connect (adjustment, "value_changed",
 			GTK_SIGNAL_FUNC (on_adjustment_value_changed), widget);
-		gtk_widget = gtk_hscale_new (GTK_ADJUSTMENT (adjustment));
-		gtk_scale_set_digits (GTK_SCALE (gtk_widget), 0);
-		gtk_range_set_update_policy (GTK_RANGE (gtk_widget),
+		gw = gtk_hscale_new (GTK_ADJUSTMENT (adjustment));
+		gtk_scale_set_digits (GTK_SCALE (gw), 
+				      get_digits (increment));
+		gtk_range_set_update_policy (GTK_RANGE (gw),
 					     GTK_UPDATE_DISCONTINUOUS);
 		if (strlen (info))
 			gtk_tooltips_set_tip (config->priv->tooltips,
-				      gtk_widget, info, NULL);
+				      gw, info, NULL);
 		else
 			gtk_tooltips_set_tip (config->priv->tooltips,
-				gtk_widget,
+				gw,
 				_("No additional information available. (4)"), NULL);
 		break;
 
 	case GP_WIDGET_MENU:
 
-		gtk_widget = gtk_combo_new ();
+		gw = gtk_combo_new ();
 		gp_widget_get_value (widget, &value_char);
 		for (i = 0; i < gp_widget_count_choices (widget); i++) {
 			gp_widget_get_choice (widget, i, &choice);
 			options = g_list_append (options, g_strdup (_(choice)));
 		}
-		gtk_combo_set_popdown_strings (GTK_COMBO (gtk_widget), options);
-		gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (gtk_widget)->entry),
+		gtk_combo_set_popdown_strings (GTK_COMBO (gw), options);
+		gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (gw)->entry),
 				    _(value_char));
-		gtk_signal_connect (GTK_OBJECT (GTK_COMBO (gtk_widget)->entry),
+		g_signal_connect (GTK_OBJECT (GTK_COMBO (gw)->entry),
 			"changed", GTK_SIGNAL_FUNC (on_entry_changed), widget);
 		break;
 
 	case GP_WIDGET_RADIO:
 
 		if (gp_widget_count_choices (widget) < 6)
-			gtk_widget = gtk_hbox_new (FALSE, 5);
+			gw = gtk_hbox_new (FALSE, 5);
 		else
-			gtk_widget = gtk_vbox_new (FALSE, 5);
+			gw = gtk_vbox_new (FALSE, 5);
 		gp_widget_get_value (widget, &value_char);
 		for (i = 0; i < gp_widget_count_choices (widget); i++) {
 			gp_widget_get_choice (widget, i, &choice);
 			button = gtk_radio_button_new_with_label (group,
 								  _(choice));
 			gtk_widget_show (button);
-			group = gtk_radio_button_group (
+			group = gtk_radio_button_get_group (
 					GTK_RADIO_BUTTON (button));
-			gtk_object_set_data (GTK_OBJECT (button), "value",
+			g_object_set_data (G_OBJECT (button), "value",
 					     (char*) choice);
-			gtk_box_pack_start (GTK_BOX (gtk_widget), button,
+			gtk_box_pack_start (GTK_BOX (gw), button,
 					    FALSE, FALSE, 0);
 			if (value_char && !strcmp (value_char, choice))
 				gtk_toggle_button_set_active (
 					GTK_TOGGLE_BUTTON (button), TRUE);
-			gtk_signal_connect (GTK_OBJECT (button), "toggled",
+			g_signal_connect (GTK_OBJECT (button), "toggled",
 				GTK_SIGNAL_FUNC (on_radio_button_toggled),
 				widget);
 			if (strlen (info))
@@ -585,19 +674,18 @@ create_widgets (GtkamConfig *config, CameraWidget *widget)
 
 	case GP_WIDGET_TOGGLE:
 
-		gtk_widget = gtk_check_button_new_with_label (label);
+		gw = gtk_check_button_new_with_label (label);
 		gp_widget_get_value (widget, &value_int);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_widget),
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gw),
 					      (value_int != 0));
-		gtk_signal_connect (GTK_OBJECT (gtk_widget), "toggled",
+		g_signal_connect (GTK_OBJECT (gw), "toggled",
 				    GTK_SIGNAL_FUNC (on_toggle_button_toggled),
 				    widget);
 		if (strlen (info))
 			gtk_tooltips_set_tip (config->priv->tooltips,
-				      gtk_widget, info, NULL);
+				      gw, info, NULL);
 		else
-			gtk_tooltips_set_tip (config->priv->tooltips,
-				gtk_widget,
+			gtk_tooltips_set_tip (config->priv->tooltips, gw,
 				_("No additional information available. (6)"), NULL);
 		break;
 
@@ -607,9 +695,9 @@ create_widgets (GtkamConfig *config, CameraWidget *widget)
 		return;
 	}
 
-	gtk_widget_show (gtk_widget);
+	gtk_widget_show (gw);
 	gtk_widget_show (frame);
-	gtk_container_add (GTK_CONTAINER (frame), gtk_widget);
+	gtk_container_add (GTK_CONTAINER (frame), gw);
 
 	gp_widget_get_parent (widget, &parent);
 	gp_widget_get_type (parent, &type);
@@ -629,7 +717,7 @@ create_widgets (GtkamConfig *config, CameraWidget *widget)
 }
 
 GtkWidget *
-gtkam_config_new (Camera *camera, gboolean multi, GtkWidget *opt_window)
+gtkam_config_new (GtkamCamera *camera)
 {
 	GtkamConfig *config;
 	GtkWidget *button, *dialog, *cancel;
@@ -638,12 +726,12 @@ gtkam_config_new (Camera *camera, gboolean multi, GtkWidget *opt_window)
 
 	g_return_val_if_fail (camera != NULL, NULL);
 
-	cancel = gtkam_cancel_new (opt_window, _("Getting configuration..."));
+	cancel = gtkam_cancel_new (_("Getting configuration..."));
 	gtk_widget_show (cancel);
-	result = gp_camera_get_config (camera, &config_widget,
+	result = gp_camera_get_config (camera->camera, &config_widget,
 		GTKAM_CANCEL (cancel)->context->context);
-	if (multi)
-		gp_camera_exit (camera, NULL);
+	if (camera->multi)
+		gp_camera_exit (camera->camera, NULL);
 	switch (result) {
 	case GP_OK:
 		break;
@@ -652,7 +740,7 @@ gtkam_config_new (Camera *camera, gboolean multi, GtkWidget *opt_window)
 		return (NULL);
 	default:
 		dialog = gtkam_error_new (result,
-			GTKAM_CANCEL (cancel)->context, opt_window,
+			GTKAM_CANCEL (cancel)->context, NULL,
 			_("Could not get configuration."));
 		gtk_widget_show (dialog);
 		gtk_object_destroy (GTK_OBJECT (cancel));
@@ -660,43 +748,37 @@ gtkam_config_new (Camera *camera, gboolean multi, GtkWidget *opt_window)
 	}
 	gtk_object_destroy (GTK_OBJECT (cancel));
 
-	config = gtk_type_new (GTKAM_TYPE_CONFIG);
-	gtk_signal_connect (GTK_OBJECT (config), "delete_event",
-			    GTK_SIGNAL_FUNC (gtk_object_destroy), NULL);
-	if (opt_window)
-		gtk_window_set_transient_for (GTK_WINDOW (config), 
-					      GTK_WINDOW (opt_window));
+	config = g_object_new (GTKAM_TYPE_CONFIG, NULL);
 
 	config->priv->camera = camera;
+	g_object_ref (G_OBJECT (camera));
 	config->priv->config = config_widget;
-	gp_camera_ref (camera);
-	config->priv->multi = multi;
 
 	config->priv->notebook = gtk_notebook_new ();
 	gtk_container_set_border_width (GTK_CONTAINER (config->priv->notebook),
 					5);
 	gtk_widget_show (config->priv->notebook);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (config)->vbox),
+	gtk_box_pack_start (GTK_BOX (GTKAM_DIALOG (config)->vbox),
 			    config->priv->notebook, TRUE, TRUE, 0);
 	create_widgets (config, config_widget);
 
-	button = gtk_button_new_with_label (_("Ok"));
+	button = gtk_button_new_from_stock (GTK_STOCK_OK);
 	gtk_widget_show (button);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+	g_signal_connect (GTK_OBJECT (button), "clicked",
 			    GTK_SIGNAL_FUNC (on_config_ok_clicked), config);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (config)->action_area),
 			   button);
 
-	button = gtk_button_new_with_label (_("Apply"));
+	button = gtk_button_new_from_stock (GTK_STOCK_APPLY);
 	gtk_widget_show (button);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+	g_signal_connect (GTK_OBJECT (button), "clicked",
 			    GTK_SIGNAL_FUNC (on_config_apply_clicked), config);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (config)->action_area),
 			   button);
 
-	button = gtk_button_new_with_label (_("Close"));
+	button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
 	gtk_widget_show (button);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+	g_signal_connect (GTK_OBJECT (button), "clicked",
 			    GTK_SIGNAL_FUNC (on_config_close_clicked), config);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (config)->action_area),
 			   button);
