@@ -45,8 +45,12 @@
 #include <gtk/gtkbutton.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkbox.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkstock.h>
 
-#include <libexif-gtk/gtk-exif-browser.h>
+#ifdef HAVE_EXIF
+#  include <libexif-gtk/gtk-exif-browser.h>
+#endif
 
 #include "gtkam-cancel.h"
 #include "gtkam-error.h"
@@ -69,50 +73,55 @@ gtkam_exif_destroy (GtkObject *object)
 }
 
 static void
-gtkam_exif_finalize (GtkObject *object)
+gtkam_exif_finalize (GObject *object)
 {
 	GtkamExif *exif = GTKAM_EXIF (object);
 
 	g_free (exif->priv);
 
-	GTK_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
-gtkam_exif_class_init (GtkamExifClass *klass)
+gtkam_exif_class_init (gpointer g_class, gpointer class_data)
 {
 	GtkObjectClass *object_class;
+	GObjectClass *gobject_class;
 
-	object_class = GTK_OBJECT_CLASS (klass);
+	object_class = GTK_OBJECT_CLASS (g_class);
 	object_class->destroy  = gtkam_exif_destroy;
-	object_class->finalize = gtkam_exif_finalize;
 
-	parent_class = gtk_type_class (PARENT_TYPE);
+	gobject_class = G_OBJECT_CLASS (g_class);
+	gobject_class->finalize = gtkam_exif_finalize;
+
+	parent_class = g_type_class_peek_parent (g_class);
 }
 
 static void
-gtkam_exif_init (GtkamExif *exif)
+gtkam_exif_init (GTypeInstance *instance, gpointer g_class)
 {
+	GtkamExif *exif = GTKAM_EXIF (instance);
+
 	exif->priv = g_new0 (GtkamExifPrivate, 1);
 }
 
-GtkType
+GType
 gtkam_exif_get_type (void)
 {
-	static GtkType exif_type = 0;
+	static GType t = 0;
+	GTypeInfo ti;
 
-	if (!exif_type) {
-		static const GtkTypeInfo exif_info = {
-			"GtkamExif",
-			sizeof (GtkamExif),
-			sizeof (GtkamExifClass),
-			(GtkClassInitFunc)  gtkam_exif_class_init,
-			(GtkObjectInitFunc) gtkam_exif_init,
-			NULL, NULL, NULL};
-		exif_type = gtk_type_unique (PARENT_TYPE, &exif_info);
+	if (!t) {
+		memset (&ti, 0, sizeof (GTypeInfo));
+		ti.class_size     = sizeof (GtkamExifClass);
+		ti.class_init     = gtkam_exif_class_init;
+		ti.instance_size  = sizeof (GtkamExif);
+		ti.instance_init  = gtkam_exif_init;
+
+		t = g_type_register_static (PARENT_TYPE, "GtkamExif", &ti, 0);
 	}
 
-	return (exif_type);
+	return (t);
 }
 
 static void
@@ -122,29 +131,34 @@ on_exif_close_clicked (GtkButton *button, GtkamExif *exif)
 }
 
 GtkWidget *
-gtkam_exif_new (Camera *camera, gboolean multi,
-		const gchar *folder, const gchar *file, GtkWidget *opt_window)
+gtkam_exif_new (GtkamCamera *camera, const gchar *folder, const gchar *file)
 {
 	GtkamExif *exif;
-	GtkWidget *button, *dialog, *browser, *c;
+	GtkWidget *button;
+#ifdef HAVE_EXIF
+	GtkWidget *dialog, *browser, *c;
 	CameraFile *cfile;
 	int result;
 	const char *data;
 	long int size;
 	ExifData *edata;
+#else
+	GtkWidget *label;
+#endif
 
-	g_return_val_if_fail (camera != NULL, NULL);
+	g_return_val_if_fail (GTKAM_IS_CAMERA (camera), NULL);
 
+#ifdef HAVE_EXIF
 	/* Get exif data */
 	gp_file_new (&cfile);
-	c = gtkam_cancel_new (opt_window,
+	c = gtkam_cancel_new (
 		_("Getting EXIF information for file '%s' in "
 		"folder '%s'..."), file, folder);
 	gtk_widget_show (c);
-	result = gp_camera_file_get (camera, folder, file, GP_FILE_TYPE_EXIF,
-			     cfile, GTKAM_CANCEL (c)->context->context);
-	if (multi)
-		gp_camera_exit (camera, NULL);
+	result = gp_camera_file_get (camera->camera, folder, file,
+		GP_FILE_TYPE_EXIF, cfile, GTKAM_CANCEL (c)->context->context);
+	if (camera->multi)
+		gp_camera_exit (camera->camera, NULL);
 	switch (result) {
 	case GP_OK:
 		break;
@@ -154,7 +168,7 @@ gtkam_exif_new (Camera *camera, gboolean multi,
 	default:
 		gp_file_unref (cfile);
 		dialog = gtkam_error_new (result, GTKAM_CANCEL (c)->context,
-			opt_window, _("Could not get exif information for "
+			NULL, _("Could not get exif information for "
 			"'%s' in folder '%s'"), file, folder);
 		gtk_widget_show (dialog);
 		gtk_object_destroy (GTK_OBJECT (c));
@@ -167,15 +181,15 @@ gtkam_exif_new (Camera *camera, gboolean multi,
 	gp_file_unref (cfile);
 	if (!edata) {
 		dialog = gtkam_error_new (GP_ERROR_CORRUPTED_DATA, NULL,
-			opt_window, _("Could not interpret EXIF data."));
+			NULL, _("Could not interpret EXIF data."));
 		gtk_widget_show (dialog);
 		return (NULL);
 	}
+#endif
 
-	exif = gtk_type_new (GTKAM_TYPE_EXIF);
-	gtk_signal_connect (GTK_OBJECT (exif), "delete_event",
-			    GTK_SIGNAL_FUNC (gtk_object_destroy), NULL);
+	exif = g_object_new (GTKAM_TYPE_EXIF, NULL);
 
+#ifdef HAVE_EXIF
 	browser = gtk_exif_browser_new ();
 	gtk_exif_browser_set_data (GTK_EXIF_BROWSER (browser), edata);
 	exif_data_unref (edata);
@@ -183,18 +197,22 @@ gtkam_exif_new (Camera *camera, gboolean multi,
 	gtk_container_set_border_width (GTK_CONTAINER (browser), 5);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (exif)->vbox), browser,
 			    TRUE, TRUE, 0);
+#else
+	label = gtk_label_new (_("Gtkam has been compiled without exif "
+		"support."));
+	gtk_widget_show (label);
+	gtk_container_set_border_width (GTK_CONTAINER (label), 5);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (exif)->vbox), label,
+			    TRUE, TRUE, 0);
+#endif
 
-	button = gtk_button_new_with_label (_("Close"));
+	button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
 	gtk_widget_show (button);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+	g_signal_connect (GTK_OBJECT (button), "clicked",
 			    GTK_SIGNAL_FUNC (on_exif_close_clicked), exif);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (exif)->action_area),
 			   button);
 	gtk_widget_grab_focus (button);
 
-	if (opt_window)
-		gtk_window_set_transient_for (GTK_WINDOW (exif),
-					      GTK_WINDOW (opt_window));
-
 	return (GTK_WIDGET (exif));
 }
