@@ -52,7 +52,8 @@
 #include <gtk/gtknotebook.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkpixmap.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkstock.h>
 
 #include "gtkam-cancel.h"
 #include "gtkam-error.h"
@@ -60,13 +61,13 @@
 
 struct _GtkamInfoPrivate
 {
-	Camera *camera;
+	GtkamCamera *camera;
 
 	CameraFileInfo info;
 	CameraFileInfo info_new;
 
-	gchar *path;
-	gchar *new_path;
+	gchar *folder;
+	gchar *name, *new_name;
 
 	GtkWidget *button_apply, *button_reset;
 	gboolean needs_update;
@@ -75,8 +76,8 @@ struct _GtkamInfoPrivate
 	GtkWidget *entry_name;
 };
 
-#define PARENT_TYPE GTK_TYPE_DIALOG
-static GtkDialogClass *parent_class;
+#define PARENT_TYPE GTKAM_TYPE_DIALOG
+static GtkamDialogClass *parent_class;
 
 enum {
 	INFO_UPDATED,
@@ -91,74 +92,86 @@ gtkam_info_destroy (GtkObject *object)
 	GtkamInfo *info = GTKAM_INFO (object);
 
 	if (info->priv->camera) {
-		gp_camera_unref (info->priv->camera);
+		g_object_unref (G_OBJECT (info->priv->camera));
 		info->priv->camera = NULL;
 	}
 
-	if (info->priv->path) {
-		g_free (info->priv->path);
-		info->priv->path = NULL;
+	if (info->priv->folder) {
+		g_free (info->priv->folder);
+		info->priv->folder = NULL;
 	}
 
-	if (info->priv->new_path) {
-		g_free (info->priv->new_path);
-		info->priv->new_path = NULL;
+	if (info->priv->new_name) {
+		g_free (info->priv->new_name);
+		info->priv->new_name = NULL;
+	}
+
+	if (info->priv->name) {
+		g_free (info->priv->name);
+		info->priv->name = NULL;
 	}
 
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
 static void
-gtkam_info_finalize (GtkObject *object)
+gtkam_info_finalize (GObject *object)
 {
 	GtkamInfo *info = GTKAM_INFO (object);
 
 	g_free (info->priv);
 
-	GTK_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
-gtkam_info_class_init (GtkamInfoClass *klass)
+gtkam_info_class_init (gpointer g_class, gpointer class_data)
 {
 	GtkObjectClass *object_class;
+	GObjectClass *gobject_class;
 
-	object_class = GTK_OBJECT_CLASS (klass);
+	object_class = GTK_OBJECT_CLASS (g_class);
 	object_class->destroy  = gtkam_info_destroy;
-	object_class->finalize = gtkam_info_finalize;
 
-	signals[INFO_UPDATED] = gtk_signal_new ("info_updated",
-		GTK_RUN_LAST, object_class->type,
-		GTK_SIGNAL_OFFSET (GtkamInfoClass, info_updated),
-		gtk_marshal_NONE__POINTER, GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
-	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
+	gobject_class = G_OBJECT_CLASS (g_class);
+	gobject_class->finalize = gtkam_info_finalize;
 
-	parent_class = gtk_type_class (PARENT_TYPE);
+	signals[INFO_UPDATED] = g_signal_new ("info_updated",
+		G_TYPE_FROM_CLASS (g_class), G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GtkamInfoClass, info_updated), NULL, NULL,
+		g_cclosure_marshal_VOID__POINTER, G_TYPE_NONE, 1,
+		G_TYPE_POINTER);
+
+	parent_class = g_type_class_peek_parent (g_class);
 }
 
 static void
-gtkam_info_init (GtkamInfo *info)
+gtkam_info_init (GTypeInstance *instance, gpointer g_class)
 {
+	GtkamInfo *info = GTKAM_INFO (instance);
+
 	info->priv = g_new0 (GtkamInfoPrivate, 1);
 }
 
-GtkType
+GType
 gtkam_info_get_type (void)
 {
-	static GtkType info_type = 0;
+	static GType type = 0;
 
-	if (!info_type) {
-		static const GtkTypeInfo info_info = {
-			"GtkamInfo",
-			sizeof (GtkamInfo),
-			sizeof (GtkamInfoClass),
-			(GtkClassInitFunc)  gtkam_info_class_init,
-			(GtkObjectInitFunc) gtkam_info_init,
-			NULL, NULL, NULL};
-		info_type = gtk_type_unique (PARENT_TYPE, &info_info);
+	if (!type) {
+		GTypeInfo ti;
+
+		memset (&ti, 0, sizeof (GTypeInfo));
+		ti.class_size     = sizeof (GtkamInfoClass);
+		ti.class_init     = gtkam_info_class_init;
+		ti.instance_size  = sizeof (GtkamInfo);
+		ti.instance_init  = gtkam_info_init;
+
+		type = g_type_register_static (PARENT_TYPE, "GtkamInfo",
+					       &ti, 0);
 	}
 
-	return (info_type);
+	return (type);
 }
 
 static void
@@ -179,25 +192,29 @@ static gboolean
 gtkam_info_update (GtkamInfo *info)
 {
 	int result;
-	gchar *dir;
 	GtkWidget *dialog, *s;
+	GtkamInfoInfoUpdatedData data;
 
 	g_return_val_if_fail (GTKAM_IS_INFO (info), FALSE);
 
-	dir = g_dirname (info->priv->path);
-
-	s = gtkam_status_new (_("Getting information for file '%s' in "
-			      "'%s'..."), dir, g_basename (info->priv->path));
+	s = gtkam_status_new (_("Setting information for file '%s' in "
+			"'%s'..."), info->priv->info, info->priv->folder);
 	gtk_widget_show (s);
-	result = gp_camera_file_set_info (info->priv->camera, dir,
-		g_basename (info->priv->path), info->priv->info_new,
+	result = gp_camera_file_set_info (info->priv->camera->camera,
+		info->priv->folder, info->priv->name, info->priv->info_new,
 		GTKAM_STATUS (s)->context->context);
-	gp_camera_file_get_info (info->priv->camera, dir,
-			g_basename (info->priv->path), &info->priv->info,
-			NULL);
-	g_free (dir);
-	gtk_signal_emit (GTK_OBJECT (info), signals[INFO_UPDATED],
-			 info->priv->path);
+	gp_camera_file_get_info (info->priv->camera->camera, 
+		info->priv->folder,
+		info->priv->new_name ? info->priv->new_name : info->priv->name,
+		&info->priv->info, NULL);
+
+	/* Emit the signal */
+	memset (&data, 0, sizeof (GtkamInfoInfoUpdatedData));
+	data.camera = info->priv->camera;
+	data.folder = info->priv->folder;
+	memcpy (&data.info, &info->priv->info, sizeof (CameraFileInfo));
+	g_signal_emit (GTK_OBJECT (info), signals[INFO_UPDATED], 0, &data);
+
 	switch (result) {
 	case GP_OK:
 		break;
@@ -208,7 +225,8 @@ gtkam_info_update (GtkamInfo *info)
 		dialog = gtkam_error_new (result, GTKAM_STATUS (s)->context,
 			GTK_WIDGET (info),
 			_("Could not set file information for "
-			"'%s'"), info->priv->path);
+			"'%s' in folder '%s'"), info->priv->name,
+			info->priv->folder);
 		gtk_widget_show (dialog);
 		gtk_object_destroy (GTK_OBJECT (s));
 		return (FALSE);
@@ -220,12 +238,11 @@ gtkam_info_update (GtkamInfo *info)
 	memset (&info->priv->info_new, 0, sizeof (CameraFileInfo));
 
 	/* Check for name change */
-	if (info->priv->new_path) {
-		g_free (info->priv->path);
-		info->priv->path = info->priv->new_path;
-		info->priv->new_path = NULL;
-		gtk_window_set_title (GTK_WINDOW (info),
-				      g_basename (info->priv->path));
+	if (info->priv->new_name) {
+		g_free (info->priv->name);
+		info->priv->name = info->priv->new_name;
+		info->priv->new_name = NULL;
+		gtk_window_set_title (GTK_WINDOW (info), info->priv->name);
 	}
 
 	return (TRUE);
@@ -255,9 +272,9 @@ on_reset_clicked (GtkButton *button, GtkamInfo *info)
 	}
 
 	memset (&info->priv->info_new, 0, sizeof (CameraFileInfo));
-	if (info->priv->new_path) {
-		g_free (info->priv->new_path);
-		info->priv->new_path = NULL;
+	if (info->priv->new_name) {
+		g_free (info->priv->new_name);
+		info->priv->new_name = NULL;
 	}
 	info->priv->needs_update = FALSE;
 	gtkam_info_update_sensitivity (info);
@@ -309,49 +326,37 @@ static void
 on_name_changed (GtkEditable *editable, GtkamInfo *info)
 {
 	const gchar *name;
-	gchar *dir;
 
 	info->priv->info_new.file.fields |= GP_FILE_INFO_NAME;
 	name = gtk_entry_get_text (GTK_ENTRY (editable));
 	strncpy (info->priv->info_new.file.name, name,
-		 sizeof (info->priv->info_new.file.name));
+		 sizeof (info->priv->info_new.file.name) - 1);
 	info->priv->needs_update = TRUE;
 	gtkam_info_update_sensitivity (info);
-
-	dir = g_dirname (info->priv->path);
-	if (info->priv->new_path)
-		g_free (info->priv->new_path);
-	if (strlen (dir) == 1)
-		info->priv->new_path = g_strdup_printf ("/%s", name);
-	else
-		info->priv->new_path = g_strdup_printf ("%s/%s", dir, name);
-	g_free (dir);
+	if (info->priv->new_name)
+		g_free (info->priv->new_name);
+	info->priv->new_name = g_strdup (name);
 }
 
 GtkWidget *
-gtkam_info_new (Camera *camera, const gchar *path, GtkWidget *opt_window)
+gtkam_info_new (GtkamCamera *camera, const gchar *folder, const gchar *name)
 {
 	GtkamInfo *info;
-	GtkWidget *button, *image, *hbox, *dialog, *notebook, *page, *label;
+	GtkWidget *button, *dialog, *notebook, *page, *label, *hbox;
 	GtkWidget *check, *entry, *c;
-	GdkPixmap *pixmap;
-	GdkBitmap *bitmap;
-	GdkPixbuf *pixbuf;
-	gchar *dir, *msg;
+	gchar *msg;
 	int result;
 	CameraFileInfo i;
 
-	g_return_val_if_fail (camera != NULL, NULL);
-	g_return_val_if_fail (path != NULL, NULL);
+	g_return_val_if_fail (GTKAM_IS_CAMERA (camera), NULL);
+	g_return_val_if_fail (folder != NULL, NULL);
+	g_return_val_if_fail (name != NULL, NULL);
 
 	/* Get file info */
-	dir = g_dirname (path);
-	c = gtkam_cancel_new (opt_window,
-		_("Getting information about file '%s' in "
-		"folder '%s'..."), g_basename (path), dir);
-	result = gp_camera_file_get_info (camera, dir, g_basename (path),
+	c = gtkam_cancel_new (_("Getting information about file '%s' in "
+		"folder '%s'..."), name, folder);
+	result = gp_camera_file_get_info (camera->camera, folder, name,
 		&i, GTKAM_CANCEL (c)->context->context);
-	g_free (dir);
 	switch (result) {
 	case GP_OK:
 		break;
@@ -360,48 +365,27 @@ gtkam_info_new (Camera *camera, const gchar *path, GtkWidget *opt_window)
 		return (NULL);
 	default:
 		dialog = gtkam_error_new (result, GTKAM_CANCEL (c)->context,
-			opt_window, _("Could not get information about file "
-			"'%s' in folder '%s'."), path);
+			NULL, _("Could not get information about file '%s' "
+			"in folder '%s'."), name, folder);
 		gtk_widget_show (dialog);
 		gtk_object_destroy (GTK_OBJECT (c));
 		return (NULL);
 	}
 	gtk_object_destroy (GTK_OBJECT (c));
 
-	info = gtk_type_new (GTKAM_TYPE_INFO);
-	gtk_signal_connect (GTK_OBJECT (info), "delete_event",
-			    GTK_SIGNAL_FUNC (gtk_object_destroy), NULL);
-	gtk_window_set_title (GTK_WINDOW (info), g_basename (path));
+	info = g_object_new (GTKAM_TYPE_INFO, NULL);
+	gtk_window_set_title (GTK_WINDOW (info), name);
 
 	info->priv->camera = camera;
-	gp_camera_ref (camera);
-	info->priv->path = g_strdup (path);
+	g_object_ref (camera);
+	info->priv->folder = g_strdup (folder);
+	info->priv->name = g_strdup (name);
 	memcpy (&info->priv->info, &i, sizeof (CameraFileInfo));
 
-	hbox = gtk_hbox_new (FALSE, 10);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (info)->vbox), hbox,
-			    TRUE, TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);
-
-	pixbuf = gdk_pixbuf_new_from_file (IMAGE_DIR "/gtkam-camera.png");
-	if (!pixbuf)
-		g_warning ("Could not load " IMAGE_DIR "/gtkam-camera.png");
-	else {
-		gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, &bitmap, 127);
-		gdk_pixbuf_unref (pixbuf);
-		image = gtk_pixmap_new (pixmap, bitmap);
-		if (pixmap)
-			gdk_pixmap_unref (pixmap);
-		if (bitmap)
-			gdk_bitmap_unref (bitmap);
-		gtk_widget_show (image);
-		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-	}
-
 	notebook = gtk_notebook_new ();
 	gtk_widget_show (notebook);
-	gtk_box_pack_start (GTK_BOX (hbox), notebook, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (GTKAM_DIALOG (info)->vbox),
+			    notebook, TRUE, TRUE, 0);
 
 	if (info->priv->info.file.fields) {
 		page = gtk_table_new (5, 2, FALSE);
@@ -423,7 +407,8 @@ gtkam_info_new (Camera *camera, const gchar *path, GtkWidget *opt_window)
 					  0, 1, 0, 1, GTK_FILL, 0, 0, 0);
 			gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
 
-			entry = gtk_entry_new_with_max_length (
+			entry = gtk_entry_new ();
+			gtk_entry_set_max_length (GTK_ENTRY (entry),
 				sizeof (info->priv->info.file.name));
 			gtk_widget_show (entry);
 			gtk_table_attach_defaults (GTK_TABLE (page), entry,
@@ -431,7 +416,7 @@ gtkam_info_new (Camera *camera, const gchar *path, GtkWidget *opt_window)
 			gtk_entry_set_text (GTK_ENTRY (entry),
 					    info->priv->info.file.name);
 			info->priv->entry_name = entry;
-			gtk_signal_connect (GTK_OBJECT (entry), "changed",
+			g_signal_connect (GTK_OBJECT (entry), "changed",
 				GTK_SIGNAL_FUNC (on_name_changed), info);
 		}
 
@@ -483,7 +468,7 @@ gtkam_info_new (Camera *camera, const gchar *path, GtkWidget *opt_window)
 			gtk_table_attach (GTK_TABLE (page), label,
 					  0, 1, 3, 4, GTK_FILL, 0, 0, 0);
 			gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
-			msg = g_strdup_printf (_("%i bytes"),
+			msg = g_strdup_printf (_("%li bytes"),
 						info->priv->info.file.size);
 			label = gtk_label_new (msg);
 			g_free (msg);
@@ -517,7 +502,7 @@ gtkam_info_new (Camera *camera, const gchar *path, GtkWidget *opt_window)
 				gtk_toggle_button_set_active (
 					GTK_TOGGLE_BUTTON (check), TRUE);
 			info->priv->check_read = check;
-			gtk_signal_connect (GTK_OBJECT (check), "toggled",
+			g_signal_connect (GTK_OBJECT (check), "toggled",
 				GTK_SIGNAL_FUNC (on_read_toggled), info);
 
 			check = gtk_check_button_new_with_label (_("Delete"));
@@ -528,7 +513,7 @@ gtkam_info_new (Camera *camera, const gchar *path, GtkWidget *opt_window)
 				gtk_toggle_button_set_active (
 					GTK_TOGGLE_BUTTON (check), TRUE);
 			info->priv->check_delete = check;
-			gtk_signal_connect (GTK_OBJECT (check), "toggled",
+			g_signal_connect (GTK_OBJECT (check), "toggled",
 				GTK_SIGNAL_FUNC (on_delete_toggled), info);
 		}
 	}
@@ -569,7 +554,7 @@ gtkam_info_new (Camera *camera, const gchar *path, GtkWidget *opt_window)
                         gtk_table_attach (GTK_TABLE (page), label,
                                           0, 1, 1, 2, GTK_FILL, 0, 0, 0);
 			gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
-                        msg = g_strdup_printf ("%i bytes",
+                        msg = g_strdup_printf ("%li bytes",
                                                 info->priv->info.preview.size);
                         label = gtk_label_new (msg);
                         g_free (msg);
@@ -617,7 +602,7 @@ gtkam_info_new (Camera *camera, const gchar *path, GtkWidget *opt_window)
                         gtk_table_attach (GTK_TABLE (page), label,
                                           0, 1, 1, 2, GTK_FILL, 0, 0, 0);
 			gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
-                        msg = g_strdup_printf ("%i bytes",
+                        msg = g_strdup_printf ("%li bytes",
                                                 info->priv->info.audio.size);
                         label = gtk_label_new (msg);
                         g_free (msg);
@@ -629,42 +614,38 @@ gtkam_info_new (Camera *camera, const gchar *path, GtkWidget *opt_window)
                 }
 	}
 
-	button = gtk_button_new_with_label (_("Ok"));
+	button = gtk_button_new_from_stock (GTK_STOCK_OK);
 	gtk_widget_show (button);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+	g_signal_connect (GTK_OBJECT (button), "clicked",
 			    GTK_SIGNAL_FUNC (on_ok_clicked), info);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (info)->action_area),
 			   button);
 	gtk_widget_grab_focus (button);
 
-	button = gtk_button_new_with_label (_("Apply"));
+	button = gtk_button_new_from_stock (GTK_STOCK_APPLY);
 	gtk_widget_show (button);
 	gtk_widget_set_sensitive (button, FALSE);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+	g_signal_connect (GTK_OBJECT (button), "clicked",
 			    GTK_SIGNAL_FUNC (on_apply_clicked), info);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (info)->action_area),
 			   button);
 	info->priv->button_apply = button;
 
-	button = gtk_button_new_with_label (_("Reset"));
+	button = gtk_button_new_from_stock (GTK_STOCK_REVERT_TO_SAVED);
 	gtk_widget_show (button);
 	gtk_widget_set_sensitive (button, FALSE);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+	g_signal_connect (GTK_OBJECT (button), "clicked",
 			    GTK_SIGNAL_FUNC (on_reset_clicked), info);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (info)->action_area),
 			   button);
 	info->priv->button_reset = button;
 
-	button = gtk_button_new_with_label (_("Cancel"));
+	button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
 	gtk_widget_show (button);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+	g_signal_connect (GTK_OBJECT (button), "clicked",
 			    GTK_SIGNAL_FUNC (on_cancel_clicked), info);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (info)->action_area),
 			   button);
 
-	if (opt_window)
-		gtk_window_set_transient_for (GTK_WINDOW (info),
-					      GTK_WINDOW (opt_window));
-
 	return (GTK_WIDGET (info));
 }
