@@ -21,6 +21,7 @@
 #ifndef __GTKAM_UTIL_H__
 #define __GTKAM_UTIL_H__
 
+#include <gphoto2/gphoto2-camera.h>
 #include <gphoto2/gphoto2-file.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtkwidget.h>
@@ -28,4 +29,4 @@
 GdkPixbuf *gdk_pixbuf_new_from_camera_file (CameraFile *file, guint max_dim,
 					    GtkWidget *opt_window);
 
-#endif __GTKAM_UTIL_H__
+#endif /* __GTKAM_UTIL_H__ */
